// File: src/handlers.rs
// Author: chmod777
// Creation Date: 2021-3-3
// License: AGPLv3

use super::State;

use html5ever::local_name;
use kuchiki::{ElementData, NodeRef};

pub fn handle_header_node(
	state: &mut State,
	importance: usize,
	node: &NodeRef,
) -> Result<&'static str, String> {
	if let Some(text_node) = node.first_child() {
		if let Some(header_text) = text_node.as_text() {
			return handle_header(state, importance, &header_text.borrow());
		} else {
			return Err(String::from("Expected text in header"));
		}
	}
	Err(String::from("Expected text node in header"))
}
pub fn handle_header(
	state: &mut State,
	importance: usize,
	header: &str,
) -> Result<&'static str, String> {
	const HEADINGS: [&'static str; 6] = ["#", "##", "###", "####", "#####", "######"];
	if importance < 1 || importance > 6 {
		return Err(String::from("Headers must have an importance of 1-6"));
	}
	state
		.gemtext
		.push_str(&state.config.empty_lines_before_h_str);
	state.gemtext.push_str(HEADINGS[importance - 1]);
	state.gemtext.push(' ');
	state.gemtext.push_str(header);
	state.gemtext.push('\n');
	state
		.gemtext
		.push_str(&state.config.empty_lines_after_h_str);

	Ok("")
}
pub fn handle_paragraph_node(state: &mut State, node: &NodeRef) -> Result<&'static str, String> {
	if let Some(text_node) = node.first_child() {
		if let Some(paragraph_text) = text_node.as_text() {
			state.gemtext.push_str(&format!(
				"{}{}\n{}",
				state.config.empty_lines_before_p_str,
				&paragraph_text.borrow(),
				state.config.empty_lines_after_p_str
			));
			Ok("")
		} else {
			Err(String::from("Expected text in text paragraph"))
		}
	} else {
		Err(String::from("Expected text node in paragraph"))
	}
}
pub fn handle_link_node(
	state: &mut State,
	element_data: &ElementData,
	node: &NodeRef,
) -> Result<&'static str, String> {
	match element_data.attributes.borrow().get(local_name!("href")) {
		Some(link) => {
			let mut link = link.to_owned();
			if let Some(ref mydomain) = state.config.convert_mydomain_links_to_gmi {
				if link.ends_with(".html") && link.contains(mydomain.as_str()) {
					link.truncate(link.len() - 4);
					link.push_str("gmi");
				}
			}
			state
				.gemtext
				.push_str(&state.config.empty_lines_before_a_str);
			state.gemtext.push_str("=> ");
			state.gemtext.push_str(&link);
		}
		None => return Err(String::from("Expected link in href on <a> element")),
	};

	if let Some(text_node) = node.first_child() {
		if let Some(link_text) = text_node.as_text() {
			state.gemtext.push(' ');
			state.gemtext.push_str(&link_text.borrow());
		}
	}

	state.gemtext.push('\n');
	state
		.gemtext
		.push_str(&state.config.empty_lines_after_a_str);

	Ok("")
}
pub fn handle_image_node(
	state: &mut State,
	element_data: &ElementData,
) -> Result<&'static str, String> {
	let attributes = element_data.attributes.borrow();
	match attributes.get(local_name!("src")) {
		Some(src) => {
			state
				.gemtext
				.push_str(&state.config.empty_lines_before_img_str);
			state.gemtext.push_str("=> ");
			state.gemtext.push_str(src);
		}
		None => return Err(String::from("Expected <img> to have src attribute")),
	};

	if let Some(title) = attributes.get(local_name!("title")) {
		state.gemtext.push(' ');
		state.gemtext.push_str(title);
	}
	state.gemtext.push('\n');

	if let Some(alt) = attributes.get(local_name!("alt")) {
		state
			.gemtext
			.push_str(&state.config.empty_lines_between_img_alt_str);
		state.gemtext.push_str(alt);
		state.gemtext.push('\n');
	}

	state
		.gemtext
		.push_str(&state.config.empty_lines_after_img_str);

	Ok("")
}
pub fn handle_list_node(state: &mut State, node: &NodeRef) -> Result<&'static str, String> {
	if let Some(child) = node.first_child() {
		if let Some(text) = child.as_text() {
			state.gemtext.push_str(&format!("* {}\n", text.borrow()));
		} else if let Some(element_data) = child.as_element() {
			return match element_data.name.local {
				local_name!("a") => {
					let _res = handle_link_node(state, element_data, &child)?;
					Ok("Recursion Handled")
				}
				local_name!("img") => {
					let _res = handle_image_node(state, element_data)?;
					Ok("Recursion Handled")
				}
				_ => Err(String::from("Unsupported nested element in li element")),
			};
		}
	}
	Ok("")
}
pub fn handle_break_node(state: &mut State, _node: &NodeRef) -> Result<&'static str, String> {
	state.gemtext.push_str(&state.config.empty_lines_br_str);
	Ok("")
}
